# @smart-gate/query-params

A lightweight, framework-agnostic package designed to help manage query parameters for filters, pagination, and other
browser state data. Seamlessly preserve complex data structures in your browser's path queries, enhancing user
experience (UX) by maintaining their preferences even as they navigate back and forth.

## Why Use This Package?

Managing query parameters can be challenging, especially with complex data structures. `@smart-gate/query-params`
simplifies this process with an easy-to-use API that works across frameworks. It offers a structured way to persist user
preferences and selections in the browser, providing a smoother user experience.

### Key Benefits

- **Simplifies Query Management:** Handle even the most complex data structures effortlessly.
- **Framework-Agnostic:** Compatible with any JavaScript framework or library.
- **Enhances User Experience:** Preserves user preferences and inputs.

## Installation

Install the package via npm:

```sh
npm install @smart-gate/query-params
```

## Sample Transformation

With `@smart-gate/query-params`, transforming and managing complex objects in the browser's query string is
straightforward. The `setQueryParams(objectToPreserve)` function enables easy setup.

**Example Input Object:**

```js
const userPreferences = {
	name: 'John',
	age: 17,
	alive: true,
	location: {
		planet: 'Earth'
	},
	hobbies: ['football', 'swimming', 'sky-diving', ['soccer'], { canWork: false, prefersToDie: true }],
	emotions: [
		{ state: 'anger', cause: 'threat' },
		{ state: 'disgust', cause: 'life' }
	]
};
```

**Transformed Query Object:**

```js
const queryParams = {
	name: 'John',
	'age()': 17,
	'alive!!': true,
	'location.planet': 'Earth',
	'hobbies[0]': 'football',
	'hobbies[1]': 'swimming',
	'hobbies[2]': 'sky-diving',
	'hobbies[3][0]': 'soccer',
	'hobbies[4].canWork!!': false,
	'hobbies[4].prefersToDie!!': true,
	'emotions[0].state': 'anger',
	'emotions[0].cause': 'threat',
	'emotions[1].state': 'disgust',
	'emotions[1].cause': 'life'
};
```

Using this structure, you can save complex objects to the browser’s query string without limitations and retrieve the
original object with `getQueryParams()`.

## Usage

`@smart-gate/query-params` is versatile and works with any framework. Here’s an example of how to use it in a Vue.js
component.

### Vue Example Component

Below is a Vue.js component that demonstrates how to use `@smart-gate/query-params` to manage filter data in the
browser's query string.

```vue
<template>
	<div class="filter-container">
		<p class="filter-title">Filter Information</p>
		<div class="input-group">
			<input v-model="filter.firstName" placeholder="First Name" />
			<input v-model="filter.lastName" placeholder="Last Name" />
			<input v-model="filter.age" placeholder="Age" type="number" />
		</div>
	</div>
</template>

<script setup>
import { onMounted, ref, watch } from 'vue';
import { getQueryParams, setQueryParams } from '@smart-gate/query-params';

const filter = ref({
	firstName: 'default value',
	lastName: null,
	age: null
});

onMounted(() => {
	filter.value = Object.assign(filter.value, { ...getQueryParams() });
});

watch(filter.value, newVal => {
	setQueryParams(newVal);
});
</script>

<style scoped>
.filter-container {
	padding: 20px;
	font-family: Arial, sans-serif;
}

.filter-title {
	font-weight: bold;
	font-size: 18px;
	margin-bottom: 10px;
}

.input-group {
	display: flex;
	gap: 10px;
	margin-bottom: 10px;
}

input {
	padding: 8px;
	border-radius: 5px;
	border: 1px solid #ccc;
}

input[type='number'] {
	width: 80px;
}
</style>
```
